"""PYNTeractA

Usage:
------

    $ pynta

Contact:
--------

- https://www.unionerenolavinosamoggia.bo.it/index.php/contatti

More information is available at:

- https://github.com/simodalla/pynteracta


Version:
--------

- pynteracta v0.4.12
"""

from .cli.commands import app

if __name__ == "__main__":
    app()



import time
from datetime import datetime, timedelta
from uuid import UUID, uuid4

from pydantic import BaseModel, Field, SecretStr, field_serializer


class JwtTokenPayload(BaseModel):
    jti: str
    aud: str
    iss: str
    iat: float
    exp: float


class JwtTokenHeaders(BaseModel):
    kid: int
    typ: str | None = None


class ServiceAccountModel(BaseModel):
    type: str = "service_account"
    private_key_id: int = Field(serialization_alias="kid")
    private_key: SecretStr
    client_id: int = Field(serialization_alias="iss")
    aud: str = "injenia/portal-authenticator"
    algorithm: str = Field("RS512", serialization_alias="alg")
    token_expiration: int = 9
    jti: UUID = Field(default_factory=uuid4)

    @field_serializer("client_id")
    def serialize_client_id(self, client_id: int, _info) -> str:
        return str(client_id)

    @property
    def jwt_token_payload(self) -> dict:
        data = self.model_dump(mode="json", include=["jti", "aud", "client_id"], by_alias=True)
        now = datetime.now()
        current_timestamp = time.mktime(now.timetuple())
        expiration_timestamp = time.mktime(
            (now + timedelta(seconds=60 * self.token_expiration)).timetuple()
        )
        data.update({"iat": current_timestamp, "exp": expiration_timestamp})
        jwt_token_payload = JwtTokenPayload(**data)
        return jwt_token_payload.model_dump(mode="json")

    @property
    def jwt_token_headers(self) -> dict:
        data = self.model_dump(mode="json", include=["private_key_id"], by_alias=True)
        jwt_headers = JwtTokenHeaders(**data).model_dump(mode="json")
        return jwt_headers



import json
import logging
from collections import OrderedDict

import jwt
import requests
from pydantic import BaseModel
from requests import Response

from . import urls
from .core import format_response_error, interactapi, mock_validate_kid
from .exceptions import (
    InteractaError,
    InteractaLoginError,
    InteractaResponseError,
    MultipleObjectsReturned,
    ObjectDoesNotFound,
    PostDoesNotFound,
)
from .schemas.models import (
    BaseListPostsElement,
    CreateCustomPostIn,
    EditCustomPostIn,
    GetCommunityDetailsResponse,
    GetCustomPostForEditResponse,
    GetPostWorkflowScreenDataForEditResponse,
    Group,
    InteractaModel,
    Post,
)
from .schemas.requests import (
    CreateGroupIn,
    CreatePostCommentRequestIn,
    CreateUserIn,
    EditGroupIn,
    EditUserIn,
    ExecutePostWorkflowOperationIn,
    GetPostDefinitionCatalogsIn,
    ListCommunityPostsIn,
    ListGroupMembersIn,
    ListSystemGroupsIn,
    ListSystemUsersIn,
)
from .schemas.responses import (
    CreateGroupOut,
    CreatePostCommentResponseOut,
    CreateUserOut,
    DeletePostResponseOut,
    EditGroupOut,
    EditUserOut,
    ExecutePostWorkflowOperationResponse,
    GetGroupForEditOut,
    GetPostDefinitionCatalogsOut,
    GetPostDefinitionOut,
    GetUserForEditOut,
    HashtagsOut,
    ListGroupMembersOut,
    ListPostDefinitionCatalogEntriesOut,
    ListSystemGroupsOut,
    ListSystemUsersElement,
    ListSystemUsersOut,
    PostCreatedOut,
    PostDetailOut,
    PostsOut,
)
from .settings import ApiSettings, InteractaSettings

logger = logging.getLogger(__name__)
jwt.api_jws.PyJWS._validate_kid = mock_validate_kid  # type: ignore


def prepare_data(data=None):
    if not data:
        return json.dumps({})
    if isinstance(data, BaseModel):
        if isinstance(data, InteractaModel):
            data = data.model_dump_json(by_alias=True)
        else:
            data = data.model_dump_json()
    else:
        data = json.dumps(data)
    return data.encode("utf-8")


class Api:
    def __init__(self, settings: ApiSettings):
        self.access_token = None
        self._log_calls = False
        self.settings = settings

    def call_post(
        self, path: str, params: dict = None, headers: dict = None, data: dict | str = None
    ):
        return self.call_api(
            "post", path=path, params=params, headers=headers, data=prepare_data(data)
        )

    def call_get(self, path: str, params: str = None, headers: dict = None):
        return self.call_api("get", path=path, params=params, headers=headers)

    def call_put(
        self, path: str, params: str = None, headers: dict = None, data: dict | str = None
    ):
        return self.call_api(
            "put", path=path, params=params, headers=headers, data=prepare_data(data)
        )

    def call_delete(self, path: str, headers: dict = None, **kwargs):
        return self.call_api("delete", path=path, headers=headers, **kwargs)

    def call_api(
        self,
        method: str,
        path: str,
        params: dict | None = None,
        headers: dict | None = None,
        data: dict | str | None = None,
        **kwargs,
    ):
        url = f"{self.settings.api_url}{path}"
        request_method = getattr(requests, method)
        if self._log_calls:
            log_msg = f"API CALL: URL [{url}] HEADERS [{headers}] DATA [{data}]"
            logger.info(log_msg)
        response = request_method(url, headers=headers, data=data, params=params, **kwargs)
        if response.status_code != 200:
            raise InteractaResponseError(format_response_error(response), response=response)
        return response


class InteractaApi(Api):
    def __init__(
        self,
        settings: InteractaSettings,
        log_calls: bool = False,
        log_call_responses: bool = False,
    ) -> None:
        self.settings = settings
        self._log_calls = log_calls
        self._log_call_responses = log_call_responses
        self._call_stack = OrderedDict()

    @property
    def authorized_header(self):
        return {
            "authorization": f"Bearer {self.access_token}",
            "content-type": "application/json",  # charset=UTF-8",
        }

    def prepare_credentials_login(self):
        login_path = urls.LOGIN_CREDENTIAL
        data = json.dumps(
            {
                "username": self.settings.auth_username,
                "password": self.settings.auth_password.get_secret_value(),
            }
        )
        return login_path, data

    def prepare_service_login(self):
        if not self.settings.auth_service_account:
            raise InteractaError("Problemi con service account")

        login_path = urls.LOGIN_SERVICE
        payload = self.settings.auth_service_account.jwt_token_payload
        headers = self.settings.auth_service_account.jwt_token_headers

        try:
            token = jwt.encode(
                payload,
                self.settings.auth_service_account.private_key.get_secret_value(),
                algorithm=self.settings.auth_service_account.algorithm,
                headers=headers,
            )
        except Exception as e:
            raise e
        data = json.dumps({"jwtAssertion": token})
        return login_path, data

    def login(self):
        try:
            path, data = (
                self.prepare_service_login()
                if self.settings.auth_service_account
                else self.prepare_credentials_login()
            )
        except Exception as e:
            raise InteractaError(f"Error on prepare login: {e}") from e

        try:
            response = self.call_api(
                "post",
                path=path,
                headers={
                    "accept": "application/json",
                    "content-type": "application/json",
                },
                data=data,
            )
        except InteractaResponseError as e:
            raise InteractaLoginError(str(e)) from e
        result = response.json()
        if "accessToken" not in result:
            raise InteractaLoginError(f"{format_response_error(response)} - No accessToken")
        self.access_token = result["accessToken"]
        return self.access_token

    @interactapi(schema_out=PostsOut)
    def list_posts(
        self,
        community_id: str | int,
        params: dict = None,
        headers: dict = None,
        data: ListCommunityPostsIn = None,
    ) -> PostsOut | Response:
        path = f"/communication/posts/data/community-list/{community_id}"
        return self.call_post(path=path, params=params, headers=headers, data=data)

    @interactapi(schema_out=PostDetailOut)
    def get_post_detail(
        self, post_id: str | int, parms: dict = None, headers: dict = None
    ) -> PostDetailOut | Response:
        path = f"/communication/posts/data/post-detail-by-id/{post_id}"
        try:
            return self.call_get(path=path, params=parms, headers=headers)
        except InteractaResponseError as e:
            raise PostDoesNotFound(f"Post with id '{post_id}' non found: {e}") from e

    @interactapi(schema_out=PostCreatedOut)
    def create_post(
        self, community_id, headers: dict = None, data: CreateCustomPostIn = None
    ) -> PostCreatedOut | Response:
        path = f"/communication/posts/manage/create-post/{community_id}"
        return self.call_post(path=path, headers=headers, data=data)

    @interactapi(schema_out=GetCustomPostForEditResponse)
    def get_post_data_for_edit(
        self, post_id, headers: dict = None
    ) -> GetCustomPostForEditResponse | Response:
        path = f"/communication/posts/manage/post-data-for-edit/{post_id}"
        return self.call_get(path=path, headers=headers)

    @interactapi()
    def edit_post(
        self,
        post_id,
        occ_token,
        headers: dict = None,
        data: EditCustomPostIn = None,
    ) -> Response:
        path = f"/communication/posts/manage/edit-post/{post_id}/{occ_token}"
        return self.call_put(path=path, headers=headers, data=data)

    @interactapi(schema_out=DeletePostResponseOut)
    def delete_post(self, post_id: int | str, headers: dict | None = None) -> DeletePostResponseOut:
        path = f"/communication/posts/manage/delete-post/{post_id}"
        return self.call_delete(path=path, headers=headers)

    @interactapi(schema_out=CreatePostCommentResponseOut)
    def create_comment_post(
        self,
        post_id: int | str,
        headers: dict = None,
        data: CreatePostCommentRequestIn | None = None,
    ) -> CreatePostCommentResponseOut | Response:
        path = f"/communication/posts/manage/create-comment/{post_id}"
        return self.call_post(path=path, headers=headers, data=data)

    @interactapi(schema_out=ListSystemUsersOut)
    def list_users(
        self, headers: dict = None, data: ListSystemUsersIn | None = None
    ) -> ListSystemUsersOut | Response:
        path = "/admin/data/users"
        return self.call_post(path=path, headers=headers, data=data)

    @interactapi(schema_out=CreateUserOut)
    def create_user(
        self, headers: dict = None, data: CreateUserIn | None = None
    ) -> CreateUserOut | Response:
        path = "/admin/manage/users"
        return self.call_post(path=path, headers=headers, data=data)

    @interactapi(schema_out=GetUserForEditOut)
    def get_user_data_for_edit(self, user_id, headers: dict = None) -> GetUserForEditOut | Response:
        path = f"/admin/manage/users/{user_id}/edit"
        return self.call_get(path=path, headers=headers)

    @interactapi(schema_out=EditUserOut)
    def edit_user(
        self, user_id, headers: dict = None, data: EditUserIn | None = None
    ) -> EditUserOut | Response:
        path = f"/admin/manage/users/{user_id}"
        return self.call_put(path=path, headers=headers, data=data)

    @interactapi()
    def delete_user(self, user_id: int, headers: dict = None) -> Response:
        path = f"/admin/manage/users/{user_id}"
        return self.call_delete(path=path, headers=headers)

    @interactapi(schema_out=ListSystemGroupsOut)
    def list_groups(
        self, headers: dict = None, data: ListSystemGroupsIn | None = None
    ) -> ListSystemGroupsOut | Response:
        path = "/admin/data/groups"
        return self.call_post(path=path, headers=headers, data=data)

    @interactapi(schema_out=CreateGroupOut)
    def create_group(
        self, headers: dict = None, data: CreateGroupIn | None = None
    ) -> CreateGroupOut | Response:
        path = "/admin/manage/groups"
        return self.call_post(path=path, headers=headers, data=data)

    @interactapi(schema_out=ListGroupMembersOut)
    def list_group_members(
        self,
        group_id: str | int,
        headers: dict = None,
        data: ListGroupMembersIn | None = None,
    ) -> ListGroupMembersOut | Response:
        path = f"/admin/data/groups/{group_id}/members"
        return self.call_post(path=path, headers=headers, data=data)

    @interactapi(schema_out=GetGroupForEditOut)
    def get_group_data_for_edit(
        self, group_id, headers: dict = None
    ) -> GetGroupForEditOut | Response:
        path = f"/admin/manage/groups/{group_id}/edit"
        return self.call_get(path=path, headers=headers)

    @interactapi(schema_out=EditGroupOut)
    def edit_group(
        self,
        group_id: str | int,
        headers: dict = None,
        data: EditGroupIn | None = None,
    ) -> EditGroupOut | Response:
        path = f"/admin/manage/groups/{group_id}"
        return self.call_put(path=path, headers=headers, data=data)

    @interactapi()
    def delete_group(self, group_id: int, headers: dict = None) -> Response:
        path = f"/admin/manage/groups/{group_id}"
        return self.call_delete(path=path, headers=headers)

    @interactapi(schema_out=HashtagsOut)
    def list_hashtags(
        self, community_id: str | int, headers: dict = None, data: dict = None
    ) -> ListSystemGroupsOut | Response:
        path = f"/admin/data/communities/{community_id}/hashtags"
        return self.call_post(path=path, headers=headers, data=data)

    @interactapi(schema_out=GetPostDefinitionOut)
    def get_post_definition_detail(
        self, community_id: str | int, headers: dict = None
    ) -> GetPostDefinitionOut | Response:
        path = f"/communication/settings/communities/{community_id}/post-definition"
        return self.call_get(path=path, headers=headers)

    @interactapi(schema_out=GetCommunityDetailsResponse)
    def get_community_detail(
        self, community_id: str | int, headers: dict = None
    ) -> GetCommunityDetailsResponse | Response:
        path = f"/communication/settings/communities/{community_id}/details"
        return self.call_get(path=path, headers=headers)

    @interactapi()
    def list_business_units(self, headers: dict = None) -> Response:
        path = "/admin/data/business-units"
        return self.call_get(path=path, headers=headers)

    @interactapi(schema_out=GetPostDefinitionCatalogsOut)
    def list_catalogs(
        self,
        catalog_ids: set[int],
        load_entries: bool = False,
        headers: dict = None,
    ) -> GetPostDefinitionCatalogsOut | Response:
        path = "/communication/settings/post-definition/catalogs"
        params = {"loadEntries": load_entries}
        data = GetPostDefinitionCatalogsIn(catalog_ids=list(catalog_ids))
        return self.call_post(path=path, headers=headers, data=data, params=params)

    @interactapi(schema_out=ListPostDefinitionCatalogEntriesOut)
    def list_catalog_entries(
        self, catalog_id: int | str, headers: dict = None, data: dict = None
    ) -> ListPostDefinitionCatalogEntriesOut | Response:
        path = f"/communication/settings/post-definition/catalogs/{catalog_id}/entries"
        return self.call_post(path=path, headers=headers, data=data)

    ### worflow operations

    @interactapi(schema_out=GetPostWorkflowScreenDataForEditResponse)
    def get_post_workflow_screen_data_for_edit(
        self, post_id: int, workflow_operation_id: int | None = None, headers: dict = None
    ) -> GetPostWorkflowScreenDataForEditResponse | Response:
        path = f"/communication/posts/manage/post-workflow-screen-data-for-edit/{post_id}"
        params = (
            {"workflowOperationId": str(workflow_operation_id)} if workflow_operation_id else None
        )
        return self.call_get(path=path, params=params, headers=headers)

    @interactapi(schema_out=ExecutePostWorkflowOperationResponse)
    def execute_post_workflow_operation(
        self,
        post_id: int,
        workflow_operation_id: int,
        headers: dict | None = None,
        data: ExecutePostWorkflowOperationIn | None = None,
    ):
        path = (
            "/communication/posts/manage/execute-post-workflow-operation/"
            f"{post_id}/{workflow_operation_id}"
        )
        return self.call_post(path=path, headers=headers, data=data)

    ### end worflow operations

    def get_post_by_title(self, community_id: int, title: str) -> Post:
        search = ListCommunityPostsIn(title=title)
        result = self.list_posts(community_id, data=search)
        posts = [post for post in result.items if title.lower() in post.title.strip().lower()]
        if len(posts) == 0:
            raise PostDoesNotFound(f"Post with '{title}' in title non found in interacta")
        elif len(posts) > 1:
            raise MultipleObjectsReturned(
                f"Multiple post with '{title}' in title founded in interacta"
            )
        return posts[0]

    def get_post_by_exact_title(self, community_id: int, title: str) -> Post | None:
        search = ListCommunityPostsIn(title=title)
        result = self.list_posts(community_id, data=search)
        posts = [post for post in result.items if title.lower() == post.title.strip().lower()]
        if len(posts) == 0:
            raise PostDoesNotFound(f"Post with title '{title}' non found in interacta")
        elif len(posts) > 1:
            raise MultipleObjectsReturned(
                f"Multiple post with title '{title}' founded in interacta"
            )
        return posts[0]

    def list_all_posts(
        self,
        community_id: str | int,
        data: ListCommunityPostsIn = None,
    ) -> list[BaseListPostsElement]:
        all_posts = []
        page_token = None
        data = data if data else ListCommunityPostsIn()
        while True:
            data.page_token = page_token
            posts = self.list_posts(
                community_id=community_id,
                data=data,
            )
            all_posts += posts.items
            if not posts.next_page_token:
                break
            page_token = posts.next_page_token
        return all_posts

    def all_users(self, data: ListSystemUsersIn | None = None) -> list[ListSystemUsersElement]:
        all_items = []
        page_token = None
        counter = 1
        data = data if data else ListSystemUsersIn(page_size=100)
        while True:
            data.page_token = page_token
            list_results = self.list_users(data=data)
            all_items += list_results.items
            if not list_results.next_page_token:
                break
            page_token = list_results.next_page_token
            counter += 1
        return all_items

    def get_group(self, name: str | None, filter: ListSystemGroupsIn | None = None) -> Group | None:
        if not filter:
            filter = ListSystemGroupsIn()
            filter.page_size = 100
            filter.calculate_total_items_count = True
            filter.full_text_filter = name
            filter.status_filter = [0]
        else:
            filter.full_text_filter = name
        result = self.list_groups(data=filter)
        groups = [group for group in result.items if group.name == name]

        if len(groups) == 0:
            raise ObjectDoesNotFound(f"Group with name '{name}' non found in interacta")
        elif len(groups) > 1:
            raise MultipleObjectsReturned(
                f"Multiple groups with name '{name}' founded in interacta"
            )
        return groups[0]

    def get_user(
        self,
        email_external_auth_service: str | None = None,
        data: ListSystemUsersIn | None = None,
    ) -> ListSystemUsersElement:
        if not data:
            data = ListSystemUsersIn()
            data.calculate_total_items_count = True
        if email_external_auth_service:
            data.external_auth_service_email_full_text_filter = email_external_auth_service

        result = self.list_users(data=data)

        json_data = data.model_dump_json(exclude_unset=True, exclude={"page_size"})
        if len(result.items) == 0:
            raise ObjectDoesNotFound(f"User with data '{json_data}' non found in interacta")
        elif len(result.items) > 1:
            raise MultipleObjectsReturned(
                f"Multiple users with data '{json_data}' founded in interacta"
            )
        return result.items[0]

    @classmethod
    def swap_type_model(cls, post: Post, PostModel: BaseModel):
        return PostModel.model_validate(post.model_dump(by_alias=True))



from enum import IntEnum, StrEnum


class FieldTypeEnum(IntEnum):
    INT = 1
    BIGINT = 2
    DECIMAL = 3
    DATE = 4
    DATETIME = 5
    STRING = 6
    ENUM = 7
    ENUM_LIST = 8
    TEXT_AREA = 9
    FLAG = 10
    DELTA_AREA = 11
    FEEDBACK = 12
    HIERARCHICAL_ENUM = 13
    LINK = 14
    GENERIC_ENTITY_LIST = 15


class FieldFilterTypeEnum(IntEnum):
    EQUAL = 1
    INTERVAL = 2
    LIKE = 3
    IN = 4
    CONTAINS = 5
    IS_NULL_OR_IN = 6
    IS_EMPTY = 7


class LoginProviderEnum(StrEnum):
    GOOGLE = "google"
    MICROSOFT = "microsoft"
    CUSTOM = "custom"



import functools

from jwt.exceptions import InvalidTokenError
from requests import Response


def mock_validate_kid(self, kid) -> None:
    if not isinstance(kid, str) and not isinstance(kid, int):
        raise InvalidTokenError("Key ID header parameter must be a string or an int")


def interactapi(func=None, *, schema_out=None):
    if func is None:
        return functools.partial(interactapi, schema_out=schema_out)

    @functools.wraps(func)
    def wrapper(self, *args, **kwargs):
        if "headers" not in kwargs:
            kwargs["headers"] = self.authorized_header
        else:
            kwargs["headers"].update(self.authorized_header)
        response = func(self, *args, **kwargs)
        if not schema_out:
            return response
        result = schema_out.model_validate(response.json())
        result._response = response
        return result

    return wrapper


def format_response_error(response: Response) -> str:
    return (
        f"url: {response.url} - response {response.status_code}"
        f" headers: {response.headers} content: {response.text}"
    )



from pydantic import ConfigDict, EmailStr

from .core import InteractaIn, InteractaModel
from .models import (
    AcknowledgeTaskFilter,
    AdminUserPreferences,
    CustomFieldFilter,
    GroupBase,
    ResetUserCustomCredentialsCommand,
    UserCredentialsConfiguration,
    UserInfo,
)


class ListCommunityPostsIn(InteractaIn):
    # ListCommunityPostsRequestDTO
    # Filtro sul titolo
    title: str | None = None
    # Filtro sulla descrizione
    description: str | None = None
    # Filtro utente creatore
    created_by_user_id: int | None = None
    # Filtro lowerbound sulla data di creazione
    creation_timestamp_from: int | None = None
    # Filtro upperbound sulla data di creazione
    creation_timestamp_to: int | None = None
    # Filtro utente ultima modifica
    modified_by_user_id: int | None = None
    # Filtro lowerbound sulla data di modifica
    modified_timestamp_from: int | None = None
    # Filtro upperbound sulla data di modifica
    modified_timestamp_to: int | None = None
    # Contenuto del post
    contains_text: str | None = None
    # Filtro post seguiti da me
    followed_by_me: bool | None = None
    # Filtro post per cui l'utente ha qualcosa da fare
    to_manage: bool | None = None
    # Filtro post per cui l'utente può effettuare una transizione di stat
    to_manage_processes: bool | None = None
    # Filtro post per cui l'utente ha dei task di tipo STANDARD non evase (scadute o da fare)
    # assegnate
    to_manage_standard_tasks: bool | None = None
    # Filtro post per cui l'utente è stato menzionato
    mentioned: bool | None = None
    # Stato del workflow del post (parametro passato per identificativo)
    current_workflow_status_ids: list[int] | None = None
    # Filtro sulla tipoloiga del post [1=CUSTOM, 2=EVENTO].
    post_type: int | None = None
    # Filtro sulla tipoloiga del post [1=CUSTOM, 2=EVENTO]
    post_types: list[int] | None = None
    # Filtro per le bozze [null=nessun filtro, 1=STANDARD_DRAFT, 2=SCHEDULED_DRAFT
    draft_type: int | None = None
    event_post_filter: dict | None = None
    # Criterio di ordinamento. Valori ammessi ['customId', 'title', 'creatorUser',
    # 'creationTimestamp', 'lastModifyUser', 'lastModifyTimestamp', 'lastModifyAndCommentTimestamp',
    #  'viewedByMeTimestamp', 'modifiedByMeTimestamp', 'commentedByMeTimestamp',
    # 'scheduled_publication', 'recency', 'customField-{id del campo custom}'].
    order_by: str | None = None
    # Filtri sugli hashtag
    hashtag_ids: list[int] | None = None
    # Indica se i filtri sugli hashtag vanno combinati in AND o in OR (default = false)
    hashtags_logical_and: bool | None = None
    # Filtri sugli hashtag degli allegati
    attachment_hashtag_ids: list[int] | None = None
    # Indica se i filtri sugli hashtag degli allegati vanno combinati in AND o in OR
    #  (default = false)
    attachment_hashtags_logical_and: bool | None = None
    # Filtro sulla visibilità pubblica/privata
    visibility: int | None = None
    custom_field_filters: list[CustomFieldFilter] | None = None
    # Filtri sui campi custom
    post_field_filters: list[CustomFieldFilter] | None = None
    # Filtri sui campi screen
    screen_field_filters: list[CustomFieldFilter] | None = None
    acknowledge_task_filter: AcknowledgeTaskFilter | None = None
    # Filtro per le bozze
    draft: bool | None = None
    # Ordinamento sui post pinnati. Default: 'true'
    pinned_first: bool | None = None


### Workflow


class ExecutePostWorkflowOperationIn(InteractaModel):
    model_config = ConfigDict(validate_assignment=True)

    # ExecutePostWorkflowOperationRequestDTO
    screen_data: dict = {}
    delta_area_format: int = 1
    screen_occ_token: int


### Utenti


class ListSystemUsersIn(InteractaIn):
    # ListSystemUsersRequestDTO
    # Filtro fulltext su nome cognome e email
    full_text_filter: str | None = None
    # Filtro sul nome dell'utente
    first_name_prefix_full_text_filter: str | None = None
    # Filtro sul cognome dell'utente
    last_name_prefix_full_text_filter: str | None = None
    # Filtro sull'email dell'utente
    email_prefix_full_text_filter: str | None = None
    # Filtro sull'email di autenticazione con servizi esterni
    external_auth_service_email_full_text_filter: str | None = None
    # Filtro sullo stato dell'utente, 0 = solo utenti attivi
    status_filter: list[int] | None = None
    # Filtro sui workspace
    workspace_ids: list[int] | None = None
    # Filtro per businessUnits
    business_unit_ids: list[int] | None = None
    # Filtro per area
    area_ids: list[int] | None = None
    # Filtro full-text per luogo
    place: str | None = None
    # Filtro per tipologia login
    login_provider_filter: list[str] | None = None
    # Id campo di ordinamento
    order_type_id: str | None = None


class UserSettingsIn(InteractaModel):
    # UserSettingsRequestDTO
    # Sezione Persone abilitata/disabilitata
    people_section_enabled: bool = True
    # Utente visibile nella sezione persone
    visible_in_people_section: bool = True
    # Visualizzazione profilo in versione ridotta/estesa
    reduced_profile: bool = False
    # Possibilità di visualizzare o meno il profilo di utenti terzi
    view_user_profiles: bool = True


class DataUserInfoIn(InteractaModel):
    id: int


class UserInfoIn(UserInfo):
    business_unit: DataUserInfoIn | None = None
    area: DataUserInfoIn | None = None
    manager: DataUserInfoIn | None = None


class UserEditBase(InteractaModel):
    firstname: str = ""
    lastname: str = ""
    contact_email: EmailStr | None = None
    private_email: EmailStr | None = None
    external_id: str | None = None
    user_preferences: AdminUserPreferences | None = None
    user_info: UserInfoIn | None = None
    user_settings: UserSettingsIn | None = None


class UserIn(UserEditBase):
    pass


class CreateUserIn(UserIn):
    # CreateUserRequestDTO
    user_credentials_configuration: UserCredentialsConfiguration | None = None
    reset_user_custom_credentials_command: ResetUserCustomCredentialsCommand | None = None


class EditUserIn(UserIn):
    # EditUserRequestDTO
    occ_token: int | None = None


### Gruppi


class ListSystemGroupsIn(InteractaIn):
    # ListSystemGroupsRequestDTO
    # Filtro fulltext su nome e email
    full_text_filter: str | None = None
    # Filtro sullo stato dell'utente
    status_filter: list[int] | None = None  # 0 == solo gruppi non eliminati
    # Filtro sui workspace
    workspace_ids: list[int] | None = None
    # Filtro per escludere i gruppi che contengono l'utente
    exclude_group_by_member_user_id: int | None = None
    # Id campo di ordinamento
    order_type_id: str | None = None


class ListGroupMembersIn(InteractaIn):
    # ListGroupMembersRequestDTO
    pass


class CreateGroupIn(GroupBase):
    # CreateGroupRequestDTO
    member_ids: list[int] | None = None


class EditGroupIn(CreateGroupIn):
    # EditGroupRequestDTO
    occ_token: int | None = None


class EditGroupMember(InteractaModel):
    # creato da rreverse engineering, non supportato da api a settembre 2023
    id: int | None = None
    occ_token: int | None = None
    add_user_ids: list[int] | None = None
    delete_user_ids: list[int] | None = None


class EditGroupMembersIn(InteractaModel):
    # creato da rreverse engineering, non supportato da api a settembre 2023
    group_members: list[EditGroupMember] | None = None


class GetPostDefinitionCatalogsIn(InteractaModel):
    # GetPostDefinitionCatalogsRequest
    catalog_ids: list[int] = []


class InputPostCommentAttachmentIn(InteractaModel):
    # InputPostCommentAttachmentDTO
    attachment_id: int | None = None
    name: str | None = None
    content_ref: str | None = None


class CreatePostCommentRequestIn(InteractaModel):
    # CreatePostCommentRequestDTO
    comment: str | None = None
    comment_format: int = 1  # Formato del commento, facoltativo (1=delta, 2=plainText, default: 1)
    client_uid: str | None = None
    attachments: InputPostCommentAttachmentIn | None = None
    parent_comment_id: int | None = None



from functools import cached_property

from pydantic import BaseModel, computed_field

from ..schemas.models import ListSystemUsersElement, User


class IdsData(BaseModel):
    ids: list[int] = []

    @computed_field
    @property
    def count(self) -> int:
        return len(self.ids)


class UsersStatsList(BaseModel):
    ids: list[int] = []
    provider_custom: list[int] = []
    provider_google: list[int] = []
    provider_microsoft: list[int] = []
    no_provider: list[int] = []
    deleted: list[int] = []
    blocked: list[int] = []
    active: list[int] = []

    @computed_field
    @cached_property
    def counts(self) -> dict:
        result = {}
        for field in list(self.model_json_schema()["properties"].keys()):
            result[field] = len(getattr(self, field))
        return result

    @classmethod
    def create_by_user_list(cls, users: list[ListSystemUsersElement]) -> "UsersStatsList":
        data = {p: [] for p in cls.model_json_schema()["properties"]}
        for user in users:
            data["ids"].append(user.id)
            for provider in user.login_providers:
                data[f"provider_{provider}"].append(user.id)
            if not user.login_providers:
                data["no_provider"].append(user.id)
            if user.blocked:
                data["blocked"].append(user.id)
            if user.deleted:
                data["deleted"].append(user.id)
            if not user.deleted and not user.blocked:
                data["active"].append(user.id)

        assert len(data["ids"]) == len(users)
        return cls(**data)


class UsersStats(BaseModel):
    provider_custom: dict[int, ListSystemUsersElement | User] = {}
    provider_google: dict[int, ListSystemUsersElement | User] = {}
    provider_microsoft: dict[int, ListSystemUsersElement | User] = {}
    no_provider: dict[int, ListSystemUsersElement | User] = {}
    deleted: dict[int, ListSystemUsersElement | User] = {}
    blocked: dict[int, ListSystemUsersElement | User] = {}
    active: dict[int, ListSystemUsersElement | User] = {}

    @computed_field
    @cached_property
    def counts(self) -> dict:
        result = {}
        for field in list(self.model_json_schema()["properties"].keys()):
            result[field] = len(getattr(self, field).keys())
        return result

    @classmethod
    def create_by_user_list(cls, users: list[ListSystemUsersElement]) -> "UsersStats":
        data = {p: {} for p in cls.model_json_schema()["properties"]}

        for user in users:
            for provider in user.login_providers:
                data[f"provider_{provider}"].update({user.id: user})
            if not user.login_providers:
                data["no_provider"].update({user.id: user})
            if user.blocked:
                data["blocked"].update({user.id: user})
            elif user.deleted:
                data["deleted"].update({user.id: user})
            else:
                data["active"].update({user.id: user})
        return cls(**data)



from pydantic import HttpUrl

from .core import InteractaOut, ItemCreatedEditedOut, PagedItemsOut
from .models import (
    BaseListPostsElement,
    Catalog,
    CatalogEntry,
    Group,
    Hashtag,
    ListSystemGroupsElement,
    ListSystemUsersElement,
    Post,
    PostComment,
    PostDefinition,
    PostDetail,
    PostWorkflowDefinitionState,
    PostWorkflowDefinitionTransition,
    Tag,
    User,
    UserInfo,
)
from .requests import UserEditBase, UserSettingsIn


class ListSystemGroupsOut(PagedItemsOut):
    # ListSystemGroupsResponseDTO
    items: list[ListSystemGroupsElement] | None = []


class ListSystemUsersOut(PagedItemsOut):
    # ListSystemUsersResponseDTO
    items: list[ListSystemUsersElement] | None


class ListUsersOut(PagedItemsOut):
    # ListUsersResponseDTO ##OK##
    items: list[User] | None


class ListGroupMembersOut(PagedItemsOut):
    # ListGroupMembersResponseDTO
    items: list[User] | None = []


class ListPostDefinitionCatalogEntriesOut(PagedItemsOut):
    # ListPostDefinitionCatalogEntriesResponseDTO
    items: list[CatalogEntry] | None = []


class PostDetailOut(InteractaOut, Post):
    # GetPostDetailResponseDTO
    current_workflow_state: PostWorkflowDefinitionState | None = None
    current_workflow_screen_data: dict | None = None
    mentions: list[User] | None = None
    comment_mentions: ListUsersOut | None = None
    watchers: list[User] | None = None
    watcher_users: list[User] | None = None
    watcher_groups: list[Group] | None = None
    hashtags: list[Hashtag] | None = None


class PostsOut(PagedItemsOut):
    # PagedListPostsResponseDTO
    items: list[BaseListPostsElement] | None = []


class HashtagsOut(PagedItemsOut):
    items: list[Hashtag] | None = []


class PostCreatedOut(InteractaOut):
    # CreatePostResponseDTO
    post_id: int
    next_occ_token: int | None = None
    post_data: PostDetail | None = None


class GetPostDefinitionOut(InteractaOut, PostDefinition):
    # -- GetPostDefinitionResponseDTO
    pass


class ExecutePostWorkflowOperationResponse(InteractaOut):
    # ExecutePostWorkflowOperationResponseDTO
    new_current_state: PostWorkflowDefinitionState | None = None
    new_screen_data: dict | None = None
    new_current_workflow_permitted_operations: list[PostWorkflowDefinitionTransition] | None = None
    new_can_edit_workflow_screen_data: bool | None = None
    new_last_modify_timestamp: int | None = None
    post_data_has_changed: bool | None = None


class CreateUserOut(InteractaOut):
    # CreateUserResponseDTO
    # ID dell'utente creato.
    user_id: int
    # Url dell'immagine utente.
    account_photo_url: HttpUrl | None = None
    # Nel caso in cui sono state create delle crenziali custom (username / password) riporta
    # la password generata.
    generated_password: list[str] | None = None
    # Nel caso in cui sono state create delle crenziali custom (username / password) indica se le
    # credenziali devono essere cambiate al prossimo login.
    expired_credentials: bool | None = None
    # Nel caso in cui sono state create delle crenziali custom (username / password) indica se è
    # stata spedita un'email di notifica.
    sent_email_notify: bool | None = None
    # Token per il controllo della conconcorrenza
    next_occ_token: int | None = None


class UserSettingsOut(UserSettingsIn):
    # UserSettingsResponseDTO
    edit_private_email_enabled: bool | None = None


class GetUserForEditOut(InteractaOut, UserEditBase):
    # GetUserForEditResponseDTO
    private_email_verified: bool | None = None
    occ_token: int | None = None
    blocked: bool | None = None
    user_settings: UserSettingsOut | None = None
    user_info: UserInfo | None = None


class EditUserOut(ItemCreatedEditedOut):
    # EditUserResponseDTO
    account_photo_url: str | None = None


class CreateGroupOut(Group, ItemCreatedEditedOut):
    # CreateGroupResponseDTO
    tags: list[dict] | None = None
    group_id: int | None = None


class EditGroupOut(ItemCreatedEditedOut):
    # EditGroupResponseDTO
    pass


class GetGroupForEditOut(InteractaOut, Group):
    # GetGroupForEditResponseDTO
    members: list[User] | None = None
    tags: list[Tag] | None = None


class GetPostDefinitionCatalogsOut(InteractaOut):
    # GetPostDefinitionCatalogsResponseDTO
    catalogs: list[Catalog] | None = None


class DeletePostResponseOut(InteractaOut):
    # DeletePostResponseDTO
    post_id: int


class CreatePostCommentResponseOut(InteractaOut):
    # CreatePostCommentResponseDTO
    comment: PostComment
